"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TokenService = void 0;
const tslib_1 = require("tslib");
const berrors_1 = require("berrors");
const crypto_1 = require("crypto");
const crypto_2 = tslib_1.__importDefault(require("crypto"));
const moment_timezone_1 = tslib_1.__importDefault(require("moment-timezone"));
const context_1 = require("@loopback/context");
const core_1 = require("@loopback/core");
const repository_1 = require("@loopback/repository");
const rest_1 = require("@loopback/rest");
const authentication_1 = require("@bleco/authentication");
const core_2 = require("@loopx/core");
const keys_1 = require("../keys");
const models_1 = require("../models");
const auth_1 = require("../modules/auth");
const providers_1 = require("../providers");
const repositories_1 = require("../repositories");
const REFRESH_TOKENS_IZE = 32;
const MS_IN_SECOND = 1000;
let TokenService = exports.TokenService = class TokenService {
    constructor(ctx, userRepo, userTenantRepo, refreshTokenRepo, getJwtPayload, jwtSigner, logger, loginActivityRepo, actorKey, userActivity) {
        this.ctx = ctx;
        this.userRepo = userRepo;
        this.userTenantRepo = userTenantRepo;
        this.refreshTokenRepo = refreshTokenRepo;
        this.getJwtPayload = getJwtPayload;
        this.jwtSigner = jwtSigner;
        this.logger = logger;
        this.loginActivityRepo = loginActivityRepo;
        this.actorKey = actorKey;
        this.userActivity = userActivity;
    }
    async createJWT(payload, authClient, loginType, tenantId) {
        var _a;
        try {
            let user;
            if (payload.user) {
                user = payload.user;
            }
            else if (payload.userId) {
                user = await this.userRepo.findById(payload.userId, {
                    include: [
                        {
                            relation: 'defaultTenant',
                        },
                    ],
                });
                if (payload.externalAuthToken && payload.externalRefreshToken) {
                    user.externalAuthToken = payload.externalAuthToken;
                    user.externalRefreshToken = payload.externalRefreshToken;
                }
            }
            else {
                // Do nothing and move ahead
            }
            if (!user) {
                throw new core_2.AuthErrors.UserDoesNotExist();
            }
            const data = await this.getJwtPayload(user, authClient, tenantId);
            const accessToken = await this.jwtSigner(data, {
                expiresIn: authClient.accessTokenExpiration,
            });
            const refreshToken = (0, crypto_1.randomBytes)(REFRESH_TOKENS_IZE).toString('hex');
            // Set refresh token into redis for later verification
            await this.refreshTokenRepo.set(refreshToken, {
                clientId: authClient.clientId,
                userId: user.id,
                username: user.username,
                accessToken,
                externalAuthToken: user.externalAuthToken,
                externalRefreshToken: user.externalRefreshToken,
                tenantId: data.tenantId,
            }, { ttl: authClient.refreshTokenExpiration * MS_IN_SECOND });
            const userTenant = await this.userTenantRepo.findOne({
                where: { userId: user.id },
            });
            if ((_a = this.userActivity) === null || _a === void 0 ? void 0 : _a.markUserActivity)
                this.markUserActivity(user, userTenant, { ...data }, loginType);
            return new auth_1.TokenResponse({
                accessToken,
                refreshToken,
                expiresIn: authClient.accessTokenExpiration,
                expiresAt: Math.floor((0, moment_timezone_1.default)().add(authClient.accessTokenExpiration, 's').toDate().getTime() / 1000),
            });
        }
        catch (error) {
            this.logger.error(error);
            if (berrors_1.BErrors.Error.prototype.isPrototypeOf.call(berrors_1.BErrors.Error.prototype, error)) {
                throw error;
            }
            else {
                throw new authentication_1.AuthenticationErrors.InvalidCredentials();
            }
        }
    }
    markUserActivity(user, userTenant, payload, loginType) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j;
        const size = 16;
        const encryptionKey = process.env.ENCRYPTION_KEY;
        if (encryptionKey) {
            const iv = crypto_2.default.randomBytes(size);
            /* encryption of IP Address */
            const cipherIp = crypto_2.default.createCipheriv('aes-256-gcm', encryptionKey, iv);
            const ip = (_d = (_b = (_a = this.ctx.request.headers['x-forwarded-for']) === null || _a === void 0 ? void 0 : _a.toString()) !== null && _b !== void 0 ? _b : (_c = this.ctx.request.socket.remoteAddress) === null || _c === void 0 ? void 0 : _c.toString()) !== null && _d !== void 0 ? _d : '';
            const encryptIp = Buffer.concat([cipherIp.update(ip, 'utf8'), cipherIp.final()]);
            const authTagIp = cipherIp.getAuthTag();
            const ipAddress = JSON.stringify({
                iv: iv.toString('hex'),
                encryptedData: encryptIp.toString('hex'),
                authTag: authTagIp.toString('hex'),
            });
            /* encryption of Payload Address */
            const cipherPayload = crypto_2.default.createCipheriv('aes-256-gcm', encryptionKey, iv);
            const activityPayload = JSON.stringify(payload);
            const encryptPayload = Buffer.concat([cipherPayload.update(activityPayload, 'utf8'), cipherPayload.final()]);
            const authTagPayload = cipherIp.getAuthTag();
            const tokenPayload = JSON.stringify({
                iv: iv.toString('hex'),
                encryptedData: encryptPayload.toString('hex'),
                authTag: authTagPayload.toString('hex'),
            });
            // make an entry to mark the users login activity
            let actor;
            let tenantId;
            if (userTenant) {
                actor = (_f = (_e = userTenant[this.actorKey]) === null || _e === void 0 ? void 0 : _e.toString()) !== null && _f !== void 0 ? _f : '0';
                tenantId = userTenant.tenantId;
            }
            else {
                actor = (_h = (_g = user['id']) === null || _g === void 0 ? void 0 : _g.toString()) !== null && _h !== void 0 ? _h : '0';
                tenantId = user.defaultTenantId;
            }
            const loginActivity = new models_1.LoginActivity({
                actor,
                tenantId,
                loginTime: new Date(),
                tokenPayload,
                loginType,
                deviceInfo: (_j = this.ctx.request.headers['user-agent']) === null || _j === void 0 ? void 0 : _j.toString(),
                ipAddress,
            });
            this.loginActivityRepo.create(loginActivity).catch(() => {
                this.logger.error(`Failed to add the login activity => ${JSON.stringify(loginActivity)}`);
            });
        }
    }
};
exports.TokenService = TokenService = tslib_1.__decorate([
    (0, core_1.injectable)({ scope: core_1.BindingScope.SINGLETON }),
    tslib_1.__param(0, context_1.inject.context()),
    tslib_1.__param(1, (0, repository_1.repository)(repositories_1.UserRepository)),
    tslib_1.__param(2, (0, repository_1.repository)(repositories_1.UserTenantRepository)),
    tslib_1.__param(3, (0, repository_1.repository)(repositories_1.RefreshTokenRepository)),
    tslib_1.__param(4, (0, context_1.inject)(keys_1.AuthServiceBindings.JWTPayloadProvider)),
    tslib_1.__param(5, (0, context_1.inject)(providers_1.AuthCodeBindings.JWT_SIGNER)),
    tslib_1.__param(6, (0, context_1.inject)(core_2.LOGGER.LOGGER_INJECT)),
    tslib_1.__param(7, (0, repository_1.repository)(repositories_1.LoginActivityRepository)),
    tslib_1.__param(8, (0, context_1.inject)(keys_1.AuthServiceBindings.ActorIdKey)),
    tslib_1.__param(9, (0, context_1.inject)(keys_1.AuthServiceBindings.MarkUserActivity, { optional: true })),
    tslib_1.__metadata("design:paramtypes", [rest_1.RequestContext,
        repositories_1.UserRepository,
        repositories_1.UserTenantRepository,
        repositories_1.RefreshTokenRepository, Function, Function, Object, repositories_1.LoginActivityRepository, String, Object])
], TokenService);
//# sourceMappingURL=token.service.js.map